# Copyright (C) 2008-2009 Open Society Institute
#               Thomas Moroz: tmoroz@sorosny.org
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License Version 2 as published
# by the Free Software Foundation.  You may not use, modify or distribute
# this program under any other version of the GNU General Public License.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

"""
Set all users' passwords as expired.
"""

import logging
import transaction

from karl.utils import find_profiles
from karlserve.instance import set_current_instance
from karlserve.log import set_subsystem


log = logging.getLogger(__name__)


def config_parser(name, subparsers, **helpers):
    parser = subparsers.add_parser(
        name,
        help="Set all users' passwords as expired (except admin).",
    )
    parser.add_argument('inst', metavar='instance', help='Karl instance.')
    parser.add_argument('url', help="URL of data source.")
    parser.set_defaults(
        func=expire_passwords,
        parser=parser,
        subsystem='update_feeds',
        only_one=True,
    )


def expire_passwords(args):
    root, closer = args.get_root(args.inst)
    set_current_instance(args.inst)
    set_subsystem('expire_passwords')
    log.info("Resetting all user passwords at %s" % args.url)
    profiles = find_profiles(root)
    for login in profiles:
        if login == 'admin':
            continue
        # expire password
        profile = profiles.get(login)
        profile.password_changed_time = None
    transaction.commit()
